@@ -4,7 +4,7 @@
 
    Author: Ilan Schnell
 */
-#define BITARRAY_VERSION  "3.1.0"
+#define BITARRAY_VERSION  "3.4.1"
 
 #ifdef STDC_HEADERS
 #  include <stddef.h>
@@ -138,13 +138,13 @@ zlc(bitarrayobject *self)       /* zlc = zeroed last char */
 static inline uint64_t
 zlw(bitarrayobject *self)       /* zlw = zeroed last word */
 {
-    const Py_ssize_t nbits = self->nbits;
-    const Py_ssize_t nw = 8 * (nbits / 64);  /* bytes in complete words */
-    const int nr = (nbits % 64) / 8;         /* complete remaining bytes */
+    const size_t nbits = self->nbits;
+    const size_t nw = (nbits / 64) * 8;   /* bytes in complete words */
+    const size_t nr = (nbits % 64) / 8;   /* complete remaining bytes */
     uint64_t res = 0;
 
-    assert(nw + nr == nbits / 8 && nw + nr <= Py_SIZE(self));
-    memcpy((char *) &res, self->ob_item + nw, (size_t) nr);
+    assert(nw + nr == nbits / 8 && 8 * (nw + nr) + nbits % 8 == nbits);
+    memcpy((char *) &res, self->ob_item + nw, nr);
     if (nbits % 8)
         *(((char *) &res) + nr) = zlc(self);
 
@@ -183,13 +183,26 @@ popcnt_64(uint64_t x)
 #endif
 }
 
+static inline int
+parity_64(uint64_t x)
+{
+#if (defined(__clang__) || defined(__GNUC__))
+    return __builtin_parityll(x);
+#else
+    int i;
+    for (i = 32; i > 0; i /= 2)
+        x ^= x >> i;
+    return x & 1;
+#endif
+}
+
 static inline uint64_t
 builtin_bswap64(uint64_t word)
 {
 #if (defined(__clang__) ||                                                 \
       (defined(__GNUC__)                                                   \
         && ((__GNUC__ >= 5) || (__GNUC__ == 4) && (__GNUC_MINOR__ >= 3))))
-    /* __builtin_bswap64() is available since GCC 4.3. */
+    /* __builtin_bswap64() is available since GCC 4.3 */
 #  define HAVE_BUILTIN_BSWAP64  1
     return __builtin_bswap64(word);
 #elif defined(_MSC_VER)
@@ -201,6 +214,18 @@ builtin_bswap64(uint64_t word)
 #endif
 }
 
+/* reverse order of first n bytes of p */
+static inline void
+swap_bytes(char *p, Py_ssize_t n)
+{
+    Py_ssize_t i, j;
+    for (i = 0, j = n - 1; i < j; i++, j--) {
+        char t = p[i];
+        p[i] = p[j];
+        p[j] = t;
+    }
+}
+
 /* Return distance [0..3] to next aligned pointer.
    While on modern compilers uint64_t pointers may be misaligned, it may
    cause problems on older ones.  Moreover, it may lead to slowdown (even
@@ -209,7 +234,7 @@ static inline int
 to_aligned(void *p)
 {
     int r = ((uintptr_t) p) % 4;
-    return r ? 4 - r : 0;
+    return (4 - r) % 4;
 }
 
 /* population count of n words starting from at uint64_t pointer w */
@@ -224,50 +249,11 @@ popcnt_words(uint64_t *w, Py_ssize_t n)
     return cnt;
 }
 
-/* adjust index in a manner consistent with the handling of normal slices */
-static inline void
-adjust_index(Py_ssize_t length, Py_ssize_t *i, Py_ssize_t step)
-{
-    if (*i < 0) {
-        *i += length;
-        if (*i < 0)
-            *i = (step < 0) ? -1 : 0;
-    }
-    else if (*i >= length) {
-        *i = (step < 0) ? length - 1 : length;
-    }
-}
-
-/* same as PySlice_AdjustIndices() which was introduced in Python 3.6.1 */
-static inline Py_ssize_t
-adjust_indices(Py_ssize_t length, Py_ssize_t *start, Py_ssize_t *stop,
-               Py_ssize_t step)
-{
-#if PY_VERSION_HEX >= 0x03060100
-    return PySlice_AdjustIndices(length, start, stop, step);
-#else
-    assert(step != 0);
-    adjust_index(length, start, step);
-    adjust_index(length, stop, step);
-    /*
-      a / b does integer division.  If either a or b is negative, the result
-      depends on the compiler (rounding can go toward 0 or negative infinity).
-      Therefore, we are careful that both a and b are always positive.
-    */
-    if (step < 0) {
-        if (*stop < *start)
-            return (*start - *stop - 1) / (-step) + 1;
-    }
-    else {
-        if (*start < *stop)
-            return (*stop - *start - 1) / step + 1;
-    }
-    return 0;
-#endif
-}
-
-/* adjust slice parameters such that step is always positive; produces
-   simpler loops over elements when their order is irrelevant */
+/* Adjust slice parameters such that step is always positive.
+   This produces simpler loops over elements when their order is irrelevant.
+   Moreover, for step = -1, we can now use set_span() in set_range() and
+   count_span() in count_range().
+*/
 static inline void
 adjust_step_positive(Py_ssize_t slicelength,
                      Py_ssize_t *start, Py_ssize_t *stop, Py_ssize_t *step)
@@ -295,7 +281,7 @@ conv_pybit(PyObject *value, int *vi)
     if (n == -1 && PyErr_Occurred())
         return 0;
 
-    if (n < 0 || n > 1) {
+    if (n >> 1) {
         PyErr_Format(PyExc_ValueError, "bit must be 0 or 1, got %zd", n);
         return 0;
     }
